{-|
Module      : Test.Database.Operations.PublishNode
Description : GarganText database tests
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX
-}

{-# LANGUAGE QuasiQuotes       #-}

module Test.Database.Operations.PublishNode where

import Prelude

import Control.Monad.Reader
import Gargantext.Core
import Gargantext.Core.Types.Individu
import Gargantext.Database.Action.User (getUserId)
import Gargantext.Database.Admin.Types.Node
import Gargantext.Database.Query.Table.Node
import Gargantext.Database.Query.Table.NodeNode (isNodeReadOnly)
import Gargantext.Database.Schema.Node (NodePoly(..))

import Test.API.Setup (createAliceAndBob)
import Test.API.UpdateList (newPrivateFolderForUser)
import Test.Database.Types
import Test.Tasty.HUnit

testGetUserRootPublicNode :: TestEnv -> Assertion
testGetUserRootPublicNode testEnv = do
  [aliceId, _bobId] <- createAliceAndBob testEnv
  alicePublicFolder <- flip runReaderT testEnv $ runTestMonad $ do
    getUserRootPublicNode aliceId
  _node_typename alicePublicFolder @?= (toDBid NodeFolderPublic)

testIsReadOnlyWorks :: TestEnv -> Assertion
testIsReadOnlyWorks testEnv = do
  alicePrivateFolderId <- newPrivateFolderForUser testEnv "alice"
  isRO <- flip runReaderT testEnv $ runTestMonad $ do
    -- Create a corpus, by default is not read only
    aliceUserId <- getUserId (UserName "alice")
    corpusId    <- insertDefaultNode NodeCorpus alicePrivateFolderId aliceUserId

    isNodeReadOnly corpusId

  isRO @?= False

  -- TODO(adn): Move the node under the public node, then
  -- we check that's public.
